@@ -28,13 +28,12 @@ from zope.app.form.browser.editview import EditView
 from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
 from zope.catalog.interfaces import ICatalog
 from zope.component import getUtility, getMultiAdapter, queryUtility
-from zope.html.field import IHtmlTextField
+from zope.intid.interfaces import IIntIds
 from zope.publisher.browser import BrowserView
 from zope.security.checker import canWrite
 from zope.security.proxy import removeSecurityProxy
 from zope.traversing.api import getParents
 from zope.traversing.browser.absoluteurl import absoluteURL
-from zope.viewlet.viewlet import JavaScriptViewlet
 
 from zc.table.column import GetterColumn
 
@@ -42,8 +41,8 @@ from schooltool.app.interfaces import ISchoolToolApplication
 from schooltool.app.browser import app
 from schooltool.app.membership import URIGroup, URIMembership
 from schooltool.intervention import InterventionGettext as _
+from schooltool.contact.interfaces import IContact
 from schooltool.course.interfaces import ISection, IInstructor
-from schooltool.gradebook.interfaces import IActivities
 from schooltool.intervention import intervention, sendmail, catalog
 from schooltool.intervention import interfaces
 from schooltool.person.interfaces import IPerson
@@ -60,9 +59,6 @@ from schooltool.table.interfaces import ITableFormatter
 from schooltool.term.interfaces import ITerm, IDateManager
 
 
-InterventionJSViewlet = JavaScriptViewlet('intervention.js')
-
-
 class InterventionNavigationViewlet(NavigationViewlet):
     """View class for rendering the Intervention tab."""
 
@@ -71,8 +67,6 @@ class InterventionNavigationViewlet(NavigationViewlet):
         return person is not None
 
     def url(self):
-        if getUtility(IDateManager).current_term is None:
-            return self.appURL() + '/no_current_term.html'
         person = IPerson(self.request.principal)
         return absoluteURL(person, self.request) + '/intervention_tab'
 
@@ -113,18 +107,20 @@ class StudentTableFormatter(SchoolToolTableFormatter):
         return (("last_name", False),)
 
     def cell_formatter(self, value, item, formatter):
-        container = intervention.getInterventionSchoolYear()
+        app = ISchoolToolApplication(None)
+        container = interfaces.IInterventionSchoolYear(app)
         url = absoluteURL(container, formatter.request)
         if item.username not in container:
-            interventionStudent = intervention.getInterventionStudent(
-                item.username, container.__name__)
+            # need to auto-vivyfy because we give the user a link to it
+            getMultiAdapter((item, ISchoolYear(container)),
+                interfaces.IInterventionStudent)
         return '<a href="%s/%s">%s</a>' % (url, item.username, value)
 
 
 class StudentFilterWidget(FilterWidget):
     """View class for rendering the student filter widget."""
 
-    template = ViewPageTemplateFile('student_filter.pt')
+    template = ViewPageTemplateFile('templates/student_filter.pt')
     parameters = ['SEARCH_STUDENT_NAME', 'WITH_GOALS_ONLY']
 
     def filter(self, items):
@@ -150,7 +146,8 @@ class StudentFilterWidget(FilterWidget):
             items = new_items
 
         if self.request.get('WITH_GOALS_ONLY'):
-            year = intervention.getInterventionSchoolYear()
+            app = ISchoolToolApplication(None)
+            year = interfaces.IInterventionSchoolYear(app)
             new_items = []
             for item in items:
                 username = item.username
@@ -205,7 +202,7 @@ class InterventionTableFormatter(IndexedTableFormatter):
             return _('Goal:')
 
     def intervention_cell_formatter(self, value, item, formatter):
-        url = absoluteURL(item.student, formatter.request)
+        url = absoluteURL(IPerson(item), formatter.request)
         year = interfaces.IInterventionSchoolYear(item)
         url += '/schoolyears/%s/%ss/%s' % (year.__name__,
                                            interfaces.IInterventionType(item),
@@ -219,15 +216,16 @@ class InterventionTableFormatter(IndexedTableFormatter):
 class InterventionFilterWidget(FilterWidget):
     """View class for rendering the intervention filter widget."""
 
-    template = ViewPageTemplateFile('filter.pt')
+    template = ViewPageTemplateFile('templates/filter.pt')
     parameters = ['SEARCH_NAME', 'GOALS_ONLY']
 
     def filter(self, items):
         catalog = ICatalog(self.context)
-        username = IPerson(self.request.principal).username
+        user_contact = IContact(IPerson(self.request.principal))
+        contact_intid = getUtility(IIntIds).getId(user_contact)
         pr_idx = catalog['persons_responsible']
         items = [item for item in items
-                 if username in pr_idx.documents_to_values[item['id']]]
+                 if contact_intid in pr_idx.documents_to_values[item['id']]]
 
         if 'CLEAR_SEARCH' in self.request:
             for parameter in self.parameters:
@@ -262,8 +260,12 @@ class InterventionStartupView(BrowserView):
     student_title = _("Search Students")
     inbox_title = _("Inbox")
 
-    def noCurrentTerm(self):
-        return queryUtility(IDateManager).current_term is None
+    def __call__(self):
+        if queryUtility(IDateManager).current_term is None:
+            template = ViewPageTemplateFile('templates/no_current_term.pt')
+            return template(self)
+        else:
+            return super(InterventionStartupView, self).__call__()
 
     def createInterventionTableFormatter(self, **kwargs):
         container = ISchoolToolApplication(None)['schooltool.interventions']
@@ -278,19 +280,6 @@ class InterventionStartupView(BrowserView):
 
     def update(self):
         self.setUpTables()
-        if 'SEARCH_STUDENT_BUTTON' in self.request:
-            url = absoluteURL(self.context, self.request)
-            url += '/search_students.html'
-            if (self.request.get('SEARCH_STUDENT_NAME') or
-                self.request.get('WITH_GOALS_ONLY')):
-                url += '?SEARCH_STUDENT_BUTTON'
-                if self.request.get('SEARCH_STUDENT_NAME'):
-                    url += '&SEARCH_STUDENT_NAME=%s' % (
-                        self.request['SEARCH_STUDENT_NAME'],)
-                if self.request.get('WITH_GOALS_ONLY'):
-                    url += '&WITH_GOALS_ONLY=%s' % (
-                        self.request['WITH_GOALS_ONLY'],)
-            self.request.response.redirect(url)
 
 
 class InterventionStudentsView(BrowserView):
@@ -299,23 +288,27 @@ class InterventionStudentsView(BrowserView):
     title = _("Search Student Interventions")
     student_title = _("Students")
 
+    def __call__(self):
+        if queryUtility(IDateManager).current_term is None:
+            template = ViewPageTemplateFile('templates/no_current_term.pt')
+            return template(self)
+        else:
+            return super(InterventionStudentsView, self).__call__()
+
     def inboxURL(self):
         return absoluteURL(self.context, self.request)
 
-    def noCurrentTerm(self):
-        return queryUtility(IDateManager).current_term is None
-
     def createStudentTableFormatter(self, **kwargs):
-        container = intervention.getInterventionSchoolYear()
+        app = ISchoolToolApplication(None)
+        container = interfaces.IInterventionSchoolYear(app)
         formatter = getMultiAdapter((container, self.request),
                                     ITableFormatter)
         formatter.setUp(**kwargs)
         return formatter
 
     def setUpTables(self):
-        if not self.noCurrentTerm():
-            self.student_table = self.createStudentTableFormatter(
-                prefix="student", batch_size=40)
+        self.student_table = self.createStudentTableFormatter(
+            prefix="student", batch_size=40)
 
     def update(self):
         self.setUpTables()
@@ -325,8 +318,8 @@ class InterventionStudentRedirectView(BrowserView):
     """View for redirecting to the no current term view."""
 
     def __call__(self):
-        url = absoluteURL(ISchoolToolApplication(None), self.request)
-        self.request.response.redirect(url + '/no_current_term.html')
+        template = ViewPageTemplateFile('templates/no_current_term.pt')
+        return template(self)
 
 
 class InterventionStudentView(object):
@@ -338,17 +331,20 @@ class InterventionStudentView(object):
 
     @property
     def isCurrentSchoolYear(self):
-        interventionSchoolYear = intervention.getInterventionSchoolYear()
+        app = ISchoolToolApplication(None)
+        interventionSchoolYear = interfaces.IInterventionSchoolYear(app)
         return interventionSchoolYear == self.context.__parent__
 
     @property
     def schoolyears(self):
         results = []
-        student = self.context.student
+        student = IPerson(self.context)
         app = ISchoolToolApplication(None)
         for year in ISchoolYearContainer(app).values():
-            interventionStudent = intervention.getInterventionStudent(
-                student.username, year.__name__)
+            interventionSchoolYear = interfaces.IInterventionSchoolYear(year)
+            interventionStudent = getMultiAdapter((student, year),
+                interfaces.IInterventionStudent)
+
             result = {
                 'url': absoluteURL(interventionStudent, self.request),
                 'title': year.title,
@@ -360,7 +356,7 @@ class InterventionStudentView(object):
         """Get the sections the student is a member of."""
         results = []
 
-        for item in self.context.student.groups:
+        for item in IPerson(self.context).groups:
             if ISection.providedBy(item):
                 results.append(item)
             else:
@@ -380,16 +376,17 @@ class InterventionStudentView(object):
                         self.request.response.redirect(year['url'])
 
         interventionURL = absoluteURL(self.context, self.request)
-        student = self.context.student
+        student = IPerson(self.context)
 
         self.studentURL = absoluteURL(student, self.request)
-        self.studentName = '%s %s' % (student.first_name, student.last_name)
+        self.studentName = '%s %s' % (IContact(student).first_name,
+                                      IContact(student).last_name)
 
         self.messages = []
         for k, v in sorted(self.context['messages'].items()):
             if v.status_change:
                 continue
-            name = intervention.convertIdToName(v.sender)
+            name = ', '.join(intervention.contactsName(v.sender))
             added = v.created
             message = {
                 'creation_date': added,
@@ -424,7 +421,7 @@ class InterventionStudentView(object):
         self.newGoalURL = interventionURL + '/goals/+/addGoal.html'
         self.allGoalsURL = interventionURL + '/goals/allGoals.html'
 
-        studentId = self.context.student.username
+        studentId = IPerson(self.context).username
         currentYear = ISchoolYear(self.context.__parent__)
         terms = {}
         for section in self.learnerOf():
@@ -433,7 +430,10 @@ class InterventionStudentView(object):
             if ISchoolYear(ITerm(section)) != currentYear:
                 continue
             sheets = terms.setdefault(ITerm(section), [])
-            for worksheet in IActivities(section).values():
+            activities = interfaces.IIndependentActivities(section, None)
+            if activities is None:
+                continue
+            for worksheet in activities.values():
                 if not worksheet.deployed:
                     continue
                 course_title = ', '.join([course.title for course in
@@ -457,7 +457,7 @@ class InterventionStudentView(object):
         for k, v in sorted(self.context['messages'].items()):
             if not v.status_change:
                 continue
-            name = intervention.convertIdToName(v.sender)
+            name = ', '.join(intervention.contactsName(v.sender))
             added = v.created.strftime('%x')
             message = {
                 'url': absoluteURL(v, self.request),
@@ -483,10 +483,15 @@ class SectionInterventionsView(object):
                          mapping={'section': self.section})
         self.schoolyear = interfaces.IInterventionSchoolYear(
             ISchoolYear(self.context))
-        worksheets = IActivities(self.context).values()
-        self.worksheets = [worksheet for worksheet in worksheets
-                           if worksheet.deployed]
-        self.activeSchoolYear = intervention.getInterventionSchoolYear()
+        activities = interfaces.IIndependentActivities(self.context, None)
+        if activities is None:
+            self.worksheets = []
+        else:
+            activities = removeSecurityProxy(activities)
+            self.worksheets = [worksheet for worksheet in activities.values()
+                               if worksheet.deployed]
+        app = ISchoolToolApplication(None)
+        self.activeSchoolYear = interfaces.IInterventionSchoolYear(app)
 
     def messageLinks(self, student):
         newMessageURL = '%s/messages/%s/+/addMessage.html' % (
@@ -572,10 +577,10 @@ class SectionInterventionsView(object):
 
         rows = []
         for student in self.context.members:
-            interventionStudent = intervention.getInterventionStudent(
-                student.username, schoolYearId=self.schoolyear.__name__)
+            interventionStudent = getMultiAdapter((student,
+                ISchoolYear(self.context)), interfaces.IInterventionStudent)
             row = {
-                'name': intervention.convertIdToName(student.username),
+                'name': intervention.contactName(IContact(student)),
                 'sortName': student.last_name + student.first_name,
                 'url': absoluteURL(interventionStudent, self.request),
                 'links': link_factory(interventionStudent),
@@ -593,7 +598,7 @@ class InterventionMessageAddView(app.BaseAddView):
         self.label = _("PLEASE REMEMBER: This information is part of the student's record.")
 
     def create(self, *args, **kw):
-        sender = self.request.principal._person.username
+        sender = IContact(self.request.principal._person)
         return self._factory(sender, *args)
 
     def nextURL(self):
@@ -611,7 +616,7 @@ class InterventionStatusMessageAddView(app.BaseAddView):
     """View for adding a message."""
 
     def create(self, *args, **kw):
-        sender = self.request.principal._person.username
+        sender = IContact(self.request.principal._person)
         kw['status_change'] = True
         return self._factory(sender, *args, **kw)
 
@@ -632,7 +637,7 @@ class InterventionMessageView(object):
     def __init__(self, context, request):
         self.context = context
         self.request = request
-        sender = intervention.convertIdToName(context.sender)
+        sender = ', '.join(intervention.contactsName(context.sender))
         added = context.created.strftime('%x')
         mapping = {'sender': sender, 'date': added}
         if context.status_change:
@@ -641,7 +646,7 @@ class InterventionMessageView(object):
         else:
             self.heading = _('Message from: ${sender} sent ${date}',
                              mapping=mapping)
-        self.recipients = ', '.join(intervention.convertIdsToNames(
+        self.recipients = ', '.join(intervention.contactsName(
             context.recipients))
 
 
@@ -655,12 +660,12 @@ class InterventionMessagesView(object):
         for k, v in sorted(self.context.items()):
             if self.skipMessage(v):
                 continue
-            name = intervention.convertIdToName(v.sender)
+            name = ', '.join(intervention.contactsName(v.sender))
             added = v.created
             heading = _('Message sent by ${sender} on ${date}',
                         mapping={'sender': name,
                                  'date': added.strftime('%x')})
-            recipients = ', '.join(intervention.convertIdsToNames(v.recipients))
+            recipients = ', '.join(intervention.contactsName(v.recipients))
             message = {
                 'creation_date': added,
                 'heading': heading,
@@ -675,7 +680,7 @@ class InterventionMessagesView(object):
 
     @property
     def heading(self):
-        name = intervention.convertIdToName(self.context.student.username)
+        name = intervention.contactName(IContact(IPerson(self.context)))
         return _('Messages regarding ${student}', mapping={'student': name})
 
 
@@ -687,7 +692,7 @@ class InterventionStatusMessagesView(InterventionMessagesView):
 
     @property
     def heading(self):
-        name = intervention.convertIdToName(self.context.student.username)
+        name = intervention.contactName(IContact(IPerson(self.context)))
         return _('Change of Status Messages regarding ${student}',
                  mapping={'student': name})
 
@@ -696,7 +701,7 @@ class InterventionGoalAddView(app.BaseAddView):
     """View for adding a goal."""
 
     def create(self, *args, **kw):
-        creator = self.request.principal._person.username
+        creator = IContact(self.request.principal._person)
         kw['creator'] = creator
         return self._factory(*args, **kw)
 
@@ -731,7 +736,7 @@ class InterventionGoalView(object):
         self.notified = ''
         if context.notified:
             self.notified = _('notification sent')
-        self.persons_responsible = ', '.join(intervention.convertIdsToNames(
+        self.persons_responsible = ', '.join(intervention.contactsName(
             context.persons_responsible))
         self.goal_met = _('No')
         if context.goal_met:
@@ -739,8 +744,8 @@ class InterventionGoalView(object):
 
     @property
     def heading(self):
-        student = removeSecurityProxy(self.context.student)
-        name = intervention.convertIdToName(student.username)
+        student = removeSecurityProxy(IPerson(self.context))
+        name = intervention.contactName(IContact(student))
         added = self.context.created.strftime('%x')
         return _('Goal ${goal_name} for ${student} added ${date}',
                  mapping={'goal_name': self.context.__name__,
@@ -761,7 +766,7 @@ class InterventionGoalsView(object):
             notified = ''
             if v.notified:
                 notified = _('notification sent')
-            persons_responsible = intervention.convertIdsToNames(
+            persons_responsible = intervention.contactsName(
                 v.persons_responsible)
             goal_met = _('No')
             if v.goal_met:
@@ -784,8 +789,8 @@ class InterventionGoalsView(object):
     @property
     def heading(self):
         return _('Goals for: ${first_name} ${last_name}',
-                 mapping={'first_name': self.context.student.first_name,
-                          'last_name': self.context.student.last_name})
+            mapping={'first_name': IContact(IPerson(self.context)).first_name,
+                     'last_name': IContact(IPerson(self.context)).last_name})
 
 
 class NotifyGoalsView(object):
@@ -798,7 +803,7 @@ class NotifyGoalsView(object):
         goalsNotified = sendmail.sendInterventionGoalNotifyEmails()
         self.goals = []
         for notified in goalsNotified:
-            name = intervention.convertIdToName(notified.student.username)
+            name = intervention.contactName(IContact(IPerson(notified)))
             goal = {
                 'text': _('${student} goal ${goal_name}',
                           mapping={'student': name,
@@ -808,19 +813,6 @@ class NotifyGoalsView(object):
             self.goals.append(goal)
 
 
-def configureFsckWidgets(view):
-    for name in view.fieldNames:
-        field = view.schema.get(name)
-        if field is not None and IHtmlTextField.implementedBy(field.__class__):
-            editor = getattr(view, '%s_widget' % name, None)
-            if editor is not None:
-                editor.editorWidth = 430
-                editor.editorHeight = 300
-                editor.toolbarConfiguration = 'schooltool'
-                url = absoluteURL(ISchoolToolApplication(None), view.request)
-                editor.configurationPath = (url + '/@@/editor_config.js')
-
-
 class MessagesCSVView(BrowserView):
 
     def __call__(self):
@@ -833,12 +825,13 @@ class MessagesCSVView(BrowserView):
             for student in year.values():
                 for message in student['messages'].values():
                     created = message.created.strftime('%Y-%m-%d')
-                    recipients = ', '.join(intervention.convertIdsToEmail(
+                    sender =  [IPerson(c).username for c in message.sender]
+                    recipients = ', '.join(intervention.contactsEmail(
                         message.recipients))
                     body = message.body.replace('\n', '\\n')
                     body = body.replace('\r', '\\r')
                     row = [year.__name__, student.__name__, created,
-                           message.sender, recipients, body]
+                           ', '.join(sender), recipients, body]
                     row = [item.encode('utf-8') for item in row]
                     writer.writerow(row)
         return csvfile.getvalue().decode('utf-8')
@@ -858,14 +851,16 @@ class GoalsCSVView(BrowserView):
             for student in year.values():
                 for goal in student['goals'].values():
                     created = goal.created.strftime('%Y-%m-%d')
+                    creator =  [IPerson(c).username for c in goal.creator]
                     timeline = goal.timeline.strftime('%Y-%m-%d')
                     goal_met = unicode(goal.goal_met)
                     notified = unicode(goal.notified)
                     persons_responsible = ', '.join(
-                        intervention.convertIdsToEmail(
+                        intervention.contactsEmail(
                         goal.persons_responsible))
-                    row = [year.__name__, student.__name__, created,
-                           goal.creator, goal.presenting_concerns, goal.goal,
+                    row = [year.__name__, student.__name__,
+                           created, ', '.join(creator),
+                           goal.presenting_concerns, goal.goal,
                            goal.strengths, goal.indicators, goal.intervention,
                            timeline, persons_responsible, goal_met,
                            goal.follow_up_notes, notified]
