@@ -20,17 +20,23 @@
 Student Intervention widgets.
 """
 from zope.browser.interfaces import IAdding 
-from zope.app.form.interfaces import MissingInputError
-from zope.app.form.browser.interfaces import IBrowserWidget, IInputWidget
+from zope.formlib.interfaces import MissingInputError
+from zope.formlib.interfaces import IBrowserWidget, IInputWidget
 from zope.app.form.browser.submit import Update
 from zope.browserpage import ViewPageTemplateFile
-from zope.component import getUtility
+from zope.component import getUtility, queryUtility
 from zope.interface import implements
 from zope.security.proxy import removeSecurityProxy
 
-from schooltool.contact.interfaces import IContactable, IContact
+from schooltool.contact.interfaces import IContactable, IContact, IUniqueFormKey
 from schooltool.contact.browser.relationship import get_relationship_title
+from schooltool.course.interfaces import ISectionContainer
 from schooltool.email.interfaces import IEmailUtility
+from schooltool.group.interfaces import IGroupContainer
+from schooltool.person.interfaces import IPerson
+from schooltool.schoolyear.interfaces import ISchoolYear
+from schooltool.term.interfaces import IDateManager
+
 from schooltool.intervention import InterventionGettext as _
 from schooltool.intervention import intervention
 
@@ -44,13 +50,13 @@ class WidgetBase(object):
     _prefix = 'field.'
     _error = ''
 
-    # See zope.app.form.interfaces.IWidget
+    # See zope.formlib.interfaces.IWidget
     name = None
     label = property(lambda self: self.context.title)
     hint = property(lambda self: self.context.description)
     visible = True
 
-    # See zope.app.form.interfaces.IInputWidget
+    # See zope.formlib.interfaces.IInputWidget
     required = property(lambda self: self.context.required)
 
     def __init__(self, field, request):
@@ -61,15 +67,15 @@ class WidgetBase(object):
         return IAdding.providedBy(self.context.context)
 
     def setRenderedValue(self, value):
-        """See zope.app.form.interfaces.IWidget"""
+        """See zope.formlib.interfaces.IWidget"""
         pass
 
     def setPrefix(self, prefix):
-        """See zope.app.form.interfaces.IWidget"""
+        """See zope.formlib.interfaces.IWidget"""
         self._prefix = prefix
 
     def applyChanges(self, content):
-        """See zope.app.form.interfaces.IInputWidget"""
+        """See zope.formlib.interfaces.IInputWidget"""
         field = self.context
         new_value = self.getInputValue()
         old_value = field.query(content, self)
@@ -79,109 +85,111 @@ class WidgetBase(object):
         return True
 
     def hasInput(self):
-        """See zope.app.form.interfaces.IInputWidget"""
+        """See zope.formlib.interfaces.IInputWidget"""
         return True
 
     def hasValidInput(self):
-        """See zope.app.form.interfaces.IInputWidget"""
+        """See zope.formlib.interfaces.IInputWidget"""
         return True
 
     def getInputValue(self):
-        """See zope.app.form.interfaces.IInputWidget"""
+        """See zope.formlib.interfaces.IInputWidget"""
         return None
 
     def error(self):
-        """See zope.app.form.browser.interfaces.IBrowserWidget"""
+        """See zope.formlib.interfaces.IBrowserWidget"""
         return self._error
 
     def __call__(self):
-        """See zope.app.form.browser.interfaces.IBrowserWidget"""
+        """See zope.formlib.interfaces.IBrowserWidget"""
         return self.template()
 
 
 class PersonListWidget(WidgetBase):
     """Person List Widget"""
 
-    template = ViewPageTemplateFile('person_list.pt')
+    template = ViewPageTemplateFile('templates/person_list.pt')
 
     def __init__(self, field, request):
         super(PersonListWidget, self).__init__(field, request)
 
+        # set student from field, view type
+        if self.isAddView():
+            self.student = IPerson(field.context.context)
+        else:
+            self.student = IPerson(field.context)
+        self.student = removeSecurityProxy(self.student)
+
+        # establish email capability and error message
         utility = getUtility(IEmailUtility)
         self.enabled = utility.enabled()
         self.message = ""
         if not self.enabled:
             self.message = _('Email server is not enabled')
         else:
-            sender = self.request.principal._person.username
-            if not intervention.convertIdToEmail(sender).strip():
+            sender = IContact(self.request.principal._person)
+            if not sender.email:
                 self.message = _('User has no email address')
                 self.enabled = False
 
-        valueList = self.getRequestValue()
-        if self.isAddView():
-            student = field.context.context.student
-        else:
-            student = field.context.student
-            if not Update in self.request:
-                valueList = getattr(field.context, field.getName())
-        student = removeSecurityProxy(student)
-
-        person_choices = intervention.getPersonsResponsible(student)
-        advisors = [removeSecurityProxy(advisor).username
-                    for advisor in student.advisors]
-
+        # build responsible, notified and changed lists of contacts
+        term = queryUtility(IDateManager).current_term
+        advisors = [IContact(removeSecurityProxy(advisor))
+                    for advisor in self.student.advisors]
+        groups = IGroupContainer(ISchoolYear(term))
+        administrators = [IContact(member)
+                          for member in groups['administrators'].members]
+        responsible = advisors + [contact for contact in administrators
+                                  if contact not in advisors]
+        for section in ISectionContainer(term).values():
+            if self.student in section.members:
+                for teacher in section.instructors:
+                    if IContact(teacher) not in responsible:
+                        responsible.append(IContact(teacher))
+        notified = [IContact(self.student)]
+        notified += [contact for contact in IContactable(self.student).contacts]
         if self.isAddView():
             changed = []
         else:
-            changed = [person
-                       for person in field.context.at_one_time_responsible
-                       if person not in person_choices]
+            changed = [contact
+                       for contact in field.context.at_one_time_responsible
+                       if contact not in responsible + notified]
 
+        # set choices attribute with complete list of contact choices
+        self.choices = responsible + notified + changed
+
+        # build value list according to view type, request
+        valueList = self.getRequestValue()
+        if not self.isAddView() and not Update in self.request:
+            valueList = getattr(field.context, field.getName())
+
+        # build final view attributes: responsible, notified and changed
         self.responsible = []
         self.notified = []
         self.changed = []
-        for choice in person_choices + changed:
+        for choice in self.choices:
             checked = ''
             if choice in valueList:
                 checked = 'checked'
 
-            value = intervention.convertIdToName(choice)
+            value = intervention.contactName(choice)
             if choice in advisors:
                 value = '%s (%s)' % (value, _('advisor'))
                 if self.isAddView() and not Update in self.request:
                     checked = 'checked'
 
-            if choice[-2] == ':':
-                personObj = intervention.convertIdToPerson(choice[:-2])
-                if personObj is None:
-                    email = None
-                else:
-                    index = int(choice[-1]) - 1
-                    if index >= len(IContactable(student).contacts):
-                        email = None
-                    else:
-                        parent = list(IContactable(student).contacts)[index]
-                        parent = removeSecurityProxy(parent)
-                        email = parent.email
-                        title = get_relationship_title(student, parent)
-                        if title is not None and len(title.strip()):
-                            value = '%s (%s)' % (value, title.lower())
-            else:
-                personObj = intervention.convertIdToPerson(choice)
-                if personObj is None:
-                    email = None
-                else:
-                    email = IContact(personObj).email
+            if choice in IContactable(self.student).contacts:
+                title = get_relationship_title(self.student, choice)
+                value = '%s (%s)' % (value, title.lower())
 
             person = {
-                'name': 'person.' + choice,
+                'name': IUniqueFormKey(choice),
                 'checked': checked,
                 'value': value,
-                'hasEmail': email is not None and len(email),
+                'hasEmail': bool(removeSecurityProxy(choice).email),
                 }
 
-            if choice == student.username or choice[-2] == ':':
+            if choice in notified:
                 self.notified.append(person)
             elif choice in changed:
                 self.changed.append(person)
@@ -192,35 +200,35 @@ class PersonListWidget(WidgetBase):
         return self.enabled and person['hasEmail']
 
     def getRequestValue(self):
-        persons = []
-        for field in self.request:
-            if field.startswith('person.'):
-                person = field[7:]
-                persons.append(person)
-        return persons
+        contacts = []
+        for contact in self.choices:
+            key = IUniqueFormKey(contact)
+            if key in self.request:
+                contacts.append(contact)
+        return contacts
 
     def setPrefix(self, prefix):
-        """See zope.app.form.interfaces.IWidget"""
+        """See zope.formlib.interfaces.IWidget"""
         self._prefix = prefix
         self.name = prefix + self.context.username
 
     def getInputValue(self):
-        """See zope.app.form.interfaces.IInputWidget"""
-        persons = self.getRequestValue()
-        if not persons and self.enabled:
+        """See zope.formlib.interfaces.IInputWidget"""
+        contacts = self.getRequestValue()
+        if not contacts and self.enabled:
             self._error = MissingInputError.__doc__
             raise MissingInputError('', '')
-        return persons
+        return contacts
 
     def hasValidInput(self):
-        """See zope.app.form.interfaces.IInputWidget"""
+        """See zope.formlib.interfaces.IInputWidget"""
         return bool(self.getInputValue())
 
 
 class GoalMetWidget(WidgetBase):
     """Goal Met Widget"""
 
-    template = ViewPageTemplateFile('goal_met.pt')
+    template = ViewPageTemplateFile('templates/goal_met.pt')
     label = property(lambda self: '')
 
     def __init__(self, field, request):
@@ -238,5 +246,5 @@ class GoalMetWidget(WidgetBase):
         self.goal_met = value and 'checked' or None
 
     def getInputValue(self):
-        """See zope.app.form.interfaces.IInputWidget"""
+        """See zope.formlib.interfaces.IInputWidget"""
         return self.request['goal_met'] == 'Yes'
